@@ -1,7 +1,7 @@
 import json
 from typing import List
 from pyspark.sql.functions import *
-from pyspark.sql.types import StructType,StructField,StringType,BooleanType,DoubleType,LongType,DecimalType, IntegerType, DateType, ShortType
+from pyspark.sql.types import StructType,StructField,StringType,BooleanType,DoubleType,LongType,DecimalType, IntegerType, DateType, ShortType, TimestampType
 from motordecalidad.constants import *
 from motordecalidad.utilities import *
 import datetime
@@ -9,7 +9,7 @@ import time
 from motordecalidad.rules import *
 
 
-print("Motor de Calidad Version Release 1.5")
+print("Motor de Calidad Version Release 1.7.21")
 
 # Main function, Invokes all the parameters from the json, Optionally filters, starts the rule validation
 # Writes and returns the summary of the validation 
@@ -21,10 +21,10 @@ def startValidation(inputspark,config,dfltPath=""):
     spark = inputspark
     dbutils = get_dbutils(spark)
     print("Inicio de validacion")
-    object,output,country,project,entity,domain,subDomain,segment,area,rules,error,filtered,dataDate = extractParamsFromJson(config)
+    object,output,country,project,entity,domain,subDomain,segment,area,rules,error,filtered,dataDate,validData,finalData,sendEmail, email = extractParamsFromJson(config)
     filteredObject = applyFilter(object,filtered)
     registerAmount = filteredObject.count()
-    validationData = validateRules(filteredObject,rules,registerAmount,entity,project,country,domain,subDomain,segment,area,error,dataDate)
+    validationData = validateRules(filteredObject,rules,registerAmount,entity,project,country,domain,subDomain,segment,area,error,dataDate,validData,finalData,sendEmail, email,output)
     writeDf(validationData, output)
     return validationData
 
@@ -41,14 +41,18 @@ def extractParamsFromJson(config):
     subDomain: str = input.get(JsonParts.SubDomain)
     segment: str = input.get(JsonParts.Segment)
     area: str = input.get(JsonParts.Area)
+    validData: str = input.get(JsonParts.ValidData)
     dataDate: str = input.get(JsonParts.DataDate)
     error = data.get(JsonParts.Error)
     filtered = data.get(JsonParts.Filter)
+    finalData = data.get(JsonParts.Data)
+    sendEmail = input.get(JsonParts.SendEmail)
+    email = input.get(JsonParts.Email)
 
     entityDf = readDf(input)
     rules = data.get(JsonParts.Rules)
     print("Extraccion de JSON completada")
-    return entityDf,output,country,project,entity,domain,subDomain,segment,area,rules,error,filtered,dataDate
+    return entityDf,output,country,project,entity,domain,subDomain,segment,area,rules,error,filtered,dataDate,validData,finalData,sendEmail,email
 
 # Function that reads the input File
 def readDf(input):
@@ -57,8 +61,7 @@ def readDf(input):
     if type == "prod_csv":
         spark.conf.set(input.get(JsonParts.Account),input.get(JsonParts.Key))        
         header = input.get(JsonParts.Header)
-        return spark.read.option("delimiter",input.get(JsonParts.Delimiter)).option("header",header).csv(str(input.get(JsonParts.Path)).format( account = dbutils.secrets.get(scope = input.get(JsonParts.Scope), key = input.get(JsonParts.Account)),
-        country = dbutils.widgets.get('country'),
+        return spark.read.option("delimiter",input.get(JsonParts.Delimiter)).option("header",header).csv(str(input.get(JsonParts.Path)).format(country = dbutils.widgets.get('country'),
         year = dbutils.widgets.get('year'),
         month = dbutils.widgets.get('month'),
         day = dbutils.widgets.get('day')))
@@ -191,6 +194,8 @@ def createErrorData(object:DataFrame) :
             columnsTypes.append(DateType())
         elif dt[One] == 'smallint' :
             columnsTypes.append(ShortType())
+        elif dt[One] == "timestamp" :
+            columnsTypes.append(TimestampType())
     columnsTypes.extend([StringType(),StringType()])
     columnsList.extend(["error","run_time"])
     schema = StructType(
@@ -198,12 +203,14 @@ def createErrorData(object:DataFrame) :
         )
     return spark.createDataFrame(spark.sparkContext.emptyRDD(), schema)
 #Function that validate rules going through the defined options
-def validateRules(object:DataFrame,rules:dict,registerAmount:int, entity: str, project:str,country: str,domain: str,subDomain: str,segment: str,area: str,error,dataDate:str):
+def validateRules(object:DataFrame,rules:dict,registerAmount:int, entity: str, project:str,country: str,domain: str,subDomain: str,segment: str,area: str,error,dataDate:str,validData:str, Data,sendEmail,email,output):
     runTime = datetime.datetime.now()
     errorData = createErrorData(object)
     rulesData:List = []
+    rulesNumber = 0
     for code in rules:
         if rules[code].get(JsonParts.Fields) not in [0,["0"],"0"] :
+            rulesNumber = rulesNumber + 1
             if code[0:3] == Rules.Pre_Requisites.code:
                 print("Inicializando regla de requisitos")
                 columns = rules[code].get(JsonParts.Fields)
@@ -488,11 +495,33 @@ def validateRules(object:DataFrame,rules:dict,registerAmount:int, entity: str, p
             pass
     if errorData.count() > Zero:
         writeDf(errorData,error)
+    try:
+        if validData == "TRUE":
+            finalData = object.join(errorData,on = Data.get(JsonParts.Fields), how = LeftAntiType)
+            writeDf(finalData,Data,rulesNumber)
+    except:
+        pass
     validationData:DataFrame = spark.createDataFrame(data = rulesData, schema = OutputDataFrameColumns)
+    try:
+        pais_widget = dbutils.widgets.get('country')
+    except:
+        pais_widget = country
+    try:
+        year_widget = dbutils.widgets.get('year')
+        month_widget = dbutils.widgets.get('month')
+        day_widget = dbutils.widgets.get('day')
+        dataDateWidget = year_widget + month_widget + day_widget
+    except:
+        dataDateWidget = dataDate
+    try:
+        emailData = validationData.groupBy("CARACTERISTICA_REGLA").mean("PORCENTAJE_CALIDAD_OK")
+        if sendEmail == "YES":
+            send_email(registerAmount,rulesNumber,output.get(JsonParts.Path),emailData,email)
+    except:
+        print("Se omite el envio de correo")
     return validationData.select(
-        DataDate.value(when(lit(dataDate).isNull(),dbutils.widgets.get('year')+ "-"  + dbutils.widgets.get('month') + "-" + dbutils.widgets.get('day')).otherwise(dataDate)),
-        CountryId.value(when(lit(country).isNull(),upper(dbutils.widgets.get('country'))).otherwise(country)),
-        Country.value(when(lit(country).isNull(),upper(dbutils.widgets.get('country'))).otherwise(country)),
+        DataDate.value(lit(dataDateWidget)),
+        CountryId.value(lit(pais_widget.upper())),
         Project.value(lit(project)),
         Entity.value(lit(entity)),
         TestedFields.column,
